// Project: Secure Text Terminal V2
// Details on:
// https://homofaciens.de/technics-physical-computing-Secure-Communication-Terminal-chapter-02_ge.htm


#include <Adafruit_GFX.h>       // include Adafruit graphics library
#include <Adafruit_ILI9341.h>   // include Adafruit ILI9341 TFT library
#include <PS2Keyboard.h>

// PS/2 keyboard pins
#define DATA_PIN 4
#define CLK_PIN  3
PS2Keyboard keyboard;

#define TFT_CS    8      // TFT CS  pin is connected to arduino pin 8
#define TFT_RST   9      // TFT RST pin is connected to arduino pin 9
#define TFT_DC    10     // TFT DC  pin is connected to arduino pin 10
// initialize ILI9341 TFT library
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC, TFT_RST);

#define TEXTMAX 172
#define MESSAGETIME 2000

unsigned long lastCharReceived;

int cur_x = 0;
int cur_y = 0;
int line_h = 26;

char text[TEXTMAX];
char decryptMessage[TEXTMAX];
int textPos = 0;
int mailToPos = 0;
int messageIsDisplayed = 0;

unsigned char readChar = 0;
int decryptKeyPos = 0;
char key[]="ThisIsNoReallySecretKeyNorIsItStrong!DontSayYouWereNotWarned!";
int keyLength = strlen(key);


byte decrypt(unsigned char a, int keyPos){
  while(keyPos > keyLength - 1){
    keyPos = keyPos - keyLength;
  }

  unsigned char b = key[keyPos];

  if(a >= b){
    return(a - b);
  }
  return(a + 255 - b);
}

byte encrypt(unsigned char a, int keyPos){
  while(keyPos > keyLength - 1){
    keyPos = keyPos - keyLength;
  }

  unsigned char b = key[keyPos];

  if(a + b < 256){
    return(a + b);
  }
  return(a + b - 255);
}


void setup() {
  lastCharReceived = 0;
  keyboard.begin(DATA_PIN, CLK_PIN);

  tft.begin();
  tft.setRotation(3);
  tft.setTextColor(ILI9341_GREEN);
  tft.fillScreen(ILI9341_BLACK);
  tft.setTextSize(3);
  cur_x = 0;
  cur_y = 0;

  tft.setCursor(0, 0);
  tft.println(text);

  textPos = 0;
  text[0] = '\0';

  decryptKeyPos = 0;
  messageIsDisplayed = 0;

  // start serial port at 115200 bps:
  Serial.begin(115200);

}


void loop(void) {

  if (Serial.available() > 0){ //if we get a valid byte, read from serial:
    if(millis() - lastCharReceived > MESSAGETIME) {
      decryptKeyPos = 0;
      tft.fillScreen(ILI9341_BLACK);
    }
    messageIsDisplayed = 1;
    lastCharReceived = millis();
    readChar = Serial.read();     //get incoming byte:
    Serial.write('r');            // send a 'r' to initiate next data from computer
    if(decryptKeyPos < TEXTMAX - 2){
      decryptMessage[decryptKeyPos] = decrypt(readChar, decryptKeyPos);
      decryptKeyPos++;
      decryptMessage[decryptKeyPos] = '\0';
      tft.setCursor(0, 0);
      tft.setTextColor(ILI9341_GREEN);
      tft.println(decryptMessage);
    }
  }


  if(keyboard.available() && millis() - lastCharReceived > MESSAGETIME) {
    byte dat = keyboard.read();
    byte val = dat - '0';

    if (messageIsDisplayed == 1){
      tft.fillScreen(ILI9341_BLACK);
      messageIsDisplayed = 0;
    }

    if(dat == 127){// Backspace
      if(textPos > 0){
        textPos--;
        text[textPos] = '\0';
        tft.fillScreen(ILI9341_BLACK);
      }
    }
    else if(dat == 27){// Esc pressed, delete message
      tft.fillScreen(ILI9341_BLACK);
      textPos = 0;
      text[textPos] = '\0';
    }
    else if(dat == 13){// Enter pressed, send encrypted message
      unsigned char a;
      tft.fillScreen(ILI9341_BLACK);
      tft.setCursor(0, 0);
      tft.setTextColor(ILI9341_RED);
      for(int i = 0; i < strlen(text); i++){
        tft.print(text[i]);
        a = encrypt(text[i], i);
        Serial.write(a);                 // send the encrypted char
        while (Serial.available() <= 0); // wait for r from computer
        readChar = Serial.read();        //discard incoming byte:
      }
      delay(2000);
      tft.fillScreen(ILI9341_BLACK);
      tft.setCursor(0, 0);
      tft.setTextColor(ILI9341_BLUE);
      tft.println("Encrypted & sent!");
      delay(2000);
      tft.fillScreen(ILI9341_BLACK);

      textPos = 0;
      text[textPos] = '\0';
    }
    else{
      if(textPos < TEXTMAX - 2){
        text[textPos] = dat;
        textPos++;
        text[textPos] = '\0';
      }
    }

    tft.setCursor(0, 0);
    tft.setTextColor(ILI9341_YELLOW);
    tft.println(text);

  }


}
